package com.sun.imageio.plugins.gif;

import javax.imageio.metadata.IIOInvalidTreeException;
import org.w3c.dom.Node;
import javax.imageio.metadata.IIOMetadata;

abstract class GIFMetadata extends IIOMetadata
{
    static final int UNDEFINED_INTEGER_VALUE = -1;
    
    protected static void fatal(Node offendingNode, String message) throws IIOInvalidTreeException {
        throw new IIOInvalidTreeException(message, offendingNode);
    }
    
    protected static String getStringAttribute(Node node, String str, String s, boolean b, String[] array) throws IIOInvalidTreeException {
        Node namedItem = node.getAttributes().getNamedItem(str);
        if (namedItem == null) {
            if (!b) {
                return s;
            }
            fatal(node, "Required attribute " + str + " not present!");
        }
        String nodeValue = namedItem.getNodeValue();
        if (array != null) {
            if (nodeValue == null) {
                fatal(node, "Null value for " + node.getNodeName() + " attribute " + str + "!");
            }
            boolean n = false;
            int length = array.length;
            int i = 0;
            while (i < length) {
                if (nodeValue.equals(array[i])) {
                    n = true;
                    break;
                }
                ++i;
            }
            if (n == 0) {
                fatal(node, "Bad value for " + node.getNodeName() + " attribute " + str + "!");
            }
        }
        return nodeValue;
    }
    
    protected static int getIntAttribute(Node node, String s, int int1, boolean b, boolean b2, int n, int n2) throws IIOInvalidTreeException {
        String stringAttribute = getStringAttribute(node, s, null, b, (String[])null);
        if (stringAttribute == null || "".equals(stringAttribute)) {
            return int1;
        }
        try {
            int1 = Integer.parseInt(stringAttribute);
        }
        catch (NumberFormatException ex) {
            fatal(node, "Bad value for " + node.getNodeName() + " attribute " + s + "!");
        }
        if (b2 && (int1 < n || int1 > n2)) {
            fatal(node, "Bad value for " + node.getNodeName() + " attribute " + s + "!");
        }
        return int1;
    }
    
    protected static float getFloatAttribute(Node node, String s, float n, boolean b) throws IIOInvalidTreeException {
        String stringAttribute = getStringAttribute(node, s, null, b, (String[])null);
        if (stringAttribute == null) {
            return n;
        }
        return Float.parseFloat(stringAttribute);
    }
    
    protected static int getIntAttribute(Node node, String s, boolean b, int n, int n2) throws IIOInvalidTreeException {
        return getIntAttribute(node, s, -1, true, b, n, n2);
    }
    
    protected static float getFloatAttribute(Node node, String s) throws IIOInvalidTreeException {
        return getFloatAttribute(node, s, -1.0f, true);
    }
    
    protected static boolean getBooleanAttribute(Node node, String s, boolean b, boolean b2) throws IIOInvalidTreeException {
        Node namedItem = node.getAttributes().getNamedItem(s);
        if (namedItem == null) {
            if (!b2) {
                return b;
            }
            fatal(node, "Required attribute " + s + " not present!");
        }
        String nodeValue = namedItem.getNodeValue();
        if (nodeValue.equals("TRUE") || nodeValue.equals("true")) {
            return true;
        }
        if (nodeValue.equals("FALSE") || nodeValue.equals("false")) {
            return false;
        }
        fatal(node, "Attribute " + s + " must be 'TRUE' or 'FALSE'!");
        return false;
    }
    
    protected static boolean getBooleanAttribute(Node node, String s) throws IIOInvalidTreeException {
        return getBooleanAttribute(node, s, false, true);
    }
    
    protected static int getEnumeratedAttribute(Node node, String s, String[] array, int n, boolean b) throws IIOInvalidTreeException {
        Node namedItem = node.getAttributes().getNamedItem(s);
        if (namedItem == null) {
            if (!b) {
                return n;
            }
            fatal(node, "Required attribute " + s + " not present!");
        }
        String nodeValue = namedItem.getNodeValue();
        int i = 0;
        while (i < array.length) {
            if (nodeValue.equals(array[i])) {
                return i;
            }
            ++i;
        }
        fatal(node, "Illegal value for attribute " + s + "!");
        return -1;
    }
    
    protected static int getEnumeratedAttribute(Node node, String s, String[] array) throws IIOInvalidTreeException {
        return getEnumeratedAttribute(node, s, array, -1, true);
    }
    
    protected static String getAttribute(Node node, String str, String s, boolean b) throws IIOInvalidTreeException {
        Node namedItem = node.getAttributes().getNamedItem(str);
        if (namedItem == null) {
            if (!b) {
                return s;
            }
            fatal(node, "Required attribute " + str + " not present!");
        }
        return namedItem.getNodeValue();
    }
    
    protected static String getAttribute(Node node, String s) throws IIOInvalidTreeException {
        return getAttribute(node, s, null, true);
    }
    
    protected GIFMetadata(boolean standardMetadataFormatSupported, String nativeMetadataFormatName, String nativeMetadataFormatClassName, String[] extraMetadataFormatNames, String[] extraMetadataFormatClassNames) {
        super(standardMetadataFormatSupported, nativeMetadataFormatName, nativeMetadataFormatClassName, extraMetadataFormatNames, extraMetadataFormatClassNames);
    }
    
    @Override
    public void mergeTree(String s, Node node) throws IIOInvalidTreeException {
        if (s.equals(this.nativeMetadataFormatName)) {
            if (node == null) {
                throw new IllegalArgumentException("root == null!");
            }
            this.mergeNativeTree(node);
        }
        else if (s.equals("javax_imageio_1.0")) {
            if (node == null) {
                throw new IllegalArgumentException("root == null!");
            }
            this.mergeStandardTree(node);
        }
        else {
            throw new IllegalArgumentException("Not a recognized format!");
        }
    }
    
    protected byte[] getColorTable(Node node, String s, boolean b, int n) throws IIOInvalidTreeException {
        byte[] array = new byte[256];
        byte[] array2 = new byte[256];
        byte[] array3 = new byte[256];
        int n2 = -1;
        Node node2 = node.getFirstChild();
        if (node2 == null) {
            fatal(node, "Palette has no entries!");
        }
        while (node2 != null) {
            if (!node2.getNodeName().equals(s)) {
                fatal(node, "Only a " + s + " may be a child of a " + node2.getNodeName() + "!");
            }
            int intAttribute = getIntAttribute(node2, "index", true, 0, 255);
            if (intAttribute > n2) {
                n2 = intAttribute;
            }
            array[intAttribute] = (byte)getIntAttribute(node2, "red", true, 0, 255);
            array2[intAttribute] = (byte)getIntAttribute(node2, "green", true, 0, 255);
            array3[intAttribute] = (byte)getIntAttribute(node2, "blue", true, 0, 255);
            node2 = node2.getNextSibling();
        }
        int n3 = n2 + 1;
        if (b && n3 != n) {
            fatal(node, "Unexpected length for palette!");
        }
        byte[] array4 = new byte[3 * n3];
        int i = 0;
        int n4 = 0;
        while (i < n3) {
            array4[n4++] = array[i];
            array4[n4++] = array2[i];
            array4[n4++] = array3[i];
            ++i;
        }
        return array4;
    }
    
    protected abstract void mergeNativeTree(Node p0) throws IIOInvalidTreeException;
    
    protected abstract void mergeStandardTree(Node p0) throws IIOInvalidTreeException;
}
